package gregapi.cover.covers;

import static gregapi.data.CS.*;
import gregapi.cover.CoverData;
import gregapi.data.LH;
import gregapi.old.Textures;
import gregapi.render.BlockTextureDefault;
import gregapi.render.BlockTextureMulti;
import gregapi.render.ITexture;
import gregapi.tileentity.delegate.DelegatorTileEntity;

import java.util.List;

import net.minecraft.entity.Entity;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidHandler;

/**
 * @author Gregorius Techneticies
 */
public class CoverPump extends AbstractCoverAttachmentController {
	@Override public boolean interceptCoverPlacement(byte aCoverSide, CoverData aData, Entity aPlayer) {return !(aData.mTileEntity instanceof IFluidHandler);}
	
	public final int mThroughput;
	
	public CoverPump(int aThroughput) {
		mThroughput = aThroughput;
	}
	
	@Override
	public long onToolClick(byte aSide, CoverData aData, String aTool, long aRemainingDurability, long aQuality, Entity aPlayer, List<String> aChatReturn, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, byte aSideClicked, float aHitX, float aHitY, float aHitZ) {
		if (aTool.equals(TOOL_screwdriver)) {
			aData.visual(aSide, (short)(aData.mVisuals[aSide]==0?1:0));
			return 1000;
		}
		return 0;
	}
	
	@Override
	public void onTickPre(byte aSide, CoverData aData, long aTimer, boolean aIsServerSide, boolean aReceivedBlockUpdate, boolean aReceivedInventoryUpdate) {
		if (aIsServerSide && aTimer % 20 == 5 && aData.mTileEntity instanceof IFluidHandler) {
			DelegatorTileEntity<IFluidHandler> tDelegator = aData.mTileEntity.getAdjacentTank(aSide);
			if (tDelegator.mTileEntity != null) {
				if (aData.mVisuals[aSide]==0) {
					FluidStack tDrained = ((IFluidHandler)aData.mTileEntity).drain(FORGE_DIR[aSide], mThroughput, F);
					if (tDrained != null) {
						tDrained.amount = tDelegator.mTileEntity.fill(FORGE_DIR[tDelegator.mSideOfTileEntity], tDrained, F);
						tDelegator.mTileEntity.fill(FORGE_DIR[tDelegator.mSideOfTileEntity], ((IFluidHandler)aData.mTileEntity).drain(FORGE_DIR[aSide], tDrained.amount, T), T);
					}
				} else {
					FluidStack tDrained = tDelegator.mTileEntity.drain(FORGE_DIR[tDelegator.mSideOfTileEntity], mThroughput, F);
					if (tDrained != null) {
						tDrained.amount = ((IFluidHandler)aData.mTileEntity).fill(FORGE_DIR[aSide], tDrained, F);
						((IFluidHandler)aData.mTileEntity).fill(FORGE_DIR[aSide], tDelegator.mTileEntity.drain(FORGE_DIR[tDelegator.mSideOfTileEntity], tDrained.amount, T), T);
					}
				}
			}
		}
	}
	
	@Override
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		super.addToolTips(aList, aStack, aF3_H);
		aList.add(LH.Chat.CYAN + "Transfers " + mThroughput + " L/sec");
		aList.add(LH.Chat.DGRAY + LH.get(LH.TOOL_TO_TOGGLE_SCREWDRIVER));
	}
	
	@Override public ITexture getCoverTextureSurface(byte aSide, CoverData aData) {return aData.mVisuals[aSide]==0?sTextureOut:sTextureIn;}
	@Override public ITexture getCoverTextureAttachment(byte aSide, CoverData aData, byte aTextureSide) {return aSide == aTextureSide ? BlockTextureMulti.get(sTextureBase, aData.mVisuals[aSide]==0?sTextureOut:sTextureIn) : aSide == OPPOSITES[aTextureSide] ? BlockTextureMulti.get(sTextureBase, aData.mVisuals[aSide]!=0?sTextureOut:sTextureIn) : sTextureBase;}
	@Override public ITexture getCoverTextureHolder(byte aSide, CoverData aData, byte aTextureSide) {return sTextureBase;}
	@Override public boolean needsVisualsSaved(byte aSide, CoverData aData) {return T;}
	@Override public boolean interceptFluidFill(byte aCoverSide, CoverData aData, byte aSide, FluidStack aFluidToFill) {return aData.mVisuals[aSide]==0;}
	@Override public boolean interceptFluidDrain(byte aCoverSide, CoverData aData, byte aSide, FluidStack aFluidToDrain) {return aData.mVisuals[aSide]!=0;}
	
	public static final ITexture
	sTextureBase = BlockTextureDefault.get(new Textures.BlockIcons.CustomIcon("machines/covers/pump/base")),
	sTextureIn = BlockTextureDefault.get(new Textures.BlockIcons.CustomIcon("machines/covers/pump/in")),
	sTextureOut = BlockTextureDefault.get(new Textures.BlockIcons.CustomIcon("machines/covers/pump/out"));
}