package gregapi.recipes;

import static gregapi.data.CS.*;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.HashSetNoNulls;
import gregapi.code.ItemStackContainer;
import gregapi.code.ItemStackMap;
import gregapi.config.Config;
import gregapi.data.CS.DirectoriesGT;
import gregapi.data.IL;
import gregapi.data.LH;
import gregapi.gui.Slot_Normal;
import gregapi.oredict.OreDictManager;
import gregapi.random.IHasWorldAndCoords;
import gregapi.recipes.maps.*;
import gregapi.tileentity.ITileEntityInventoryGUI;
import gregapi.util.UT;
import gregapi.util.UT.Fluids;

import java.util.*;

import net.minecraft.item.ItemStack;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidStack;

/**
 * @author Gregorius Techneticies
 */
public class Recipe {
	public static class RecipeMap implements Runnable {
		/** RecipeMap-HashMap so that Machines can store their corresponding Recipe Lists as String NBT. */
		public static final Map<String, RecipeMap> RECIPE_MAPS = new HashMap<String, Recipe.RecipeMap>();
		
		public static final RecipeMap		sOreWasherRecipes							= new RecipeMapOreWasher			(null, "ic.recipe.orewasher"					, "Ore Washer"						, "ic2.blockOreWashingPlant"	, 0, 1, RES_PATH_GUI+"machines/OreWasher"						,/*IN-OUT-MIN-ITEM=*/ 1, 3, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sThermalCentrifugeRecipes					= new RecipeMapThermalCentrifuge	(null, "ic.recipe.thermalcentrifuge"			, "Thermal Centrifuge"				, "ic2.blockCentrifuge"			, 0, 1, RES_PATH_GUI+"machines/ThermalCentrifuge"				,/*IN-OUT-MIN-ITEM=*/ 1, 3, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 2, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sMaceratorRecipes							= new RecipeMapMacerator			(null, "ic.recipe.macerator"					, "Macerator"						, "ic2.macerator"				, 0, 1, RES_PATH_GUI+"machines/Macerator"						,/*IN-OUT-MIN-ITEM=*/ 1, 4, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sIC2CompressorRecipes						= new RecipeMapCompressor			(null, "ic.recipe.compressor"					, "Compressor"						, "ic2.compressor"				, 0, 1, RES_PATH_GUI+"machines/Compressor"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sExtractorRecipes							= new RecipeMapExtractor			(null, "ic.recipe.extractor"					, "Extractor"						, "ic2.extractor"				, 0, 1, RES_PATH_GUI+"machines/Extractor"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sRecyclerRecipes							= new RecipeMapRecycler				(null, "ic.recipe.recycler"						, "Recycler"						, "ic2.recycler"				, 0, 1, RES_PATH_GUI+"machines/Recycler"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sFurnaceRecipes								= new RecipeMapFurnace				(null, "mc.recipe.furnace"						, "Furnace"							, "smelting"					, 0, 1, RES_PATH_GUI+"machines/E_Furnace"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sMicrowaveRecipes							= new RecipeMapMicrowave			(null, "gt.recipe.microwave"					, "Microwave"						, "smelting"					, 0, 1, RES_PATH_GUI+"machines/E_Furnace"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sFurnaceFuel								= new RecipeMapFurnaceFuel			(null, "mc.recipe.furnacefuel"					, "Furnace Fuels"					, "smelting"					, 0, 1, RES_PATH_GUI+"machines/E_Furnace"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, F);
		
		public static final RecipeMap		sScannerFakeRecipes							= new RecipeMap						(null, "gt.recipe.scanner"						, "Scanner"							, null							, 0, 1, RES_PATH_GUI+"machines/Scanner"							,/*IN-OUT-MIN-ITEM=*/ 2, 2, 1,/*IN-OUT-MIN-FLUID=*/ 1, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sRockBreakerFakeRecipes						= new RecipeMap						(null, "gt.recipe.rockbreaker"					, "Rock Breaker"					, null							, 0, 1, RES_PATH_GUI+"machines/RockBreaker"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sByProductList								= new RecipeMap						(null, "gt.recipe.byproductlist"				, "Ore Byproduct List"				, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 6, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, F, T);
		public static final RecipeMap		sCrucibleSmelting							= new RecipeMap						(null, "gt.recipe.cruciblesmelting"				, "Crucible Smelting"				, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 6, 6, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Temperature: "		,    1, " K"	, T, T, F, T);
		public static final RecipeMap		sCrucibleAlloying							= new RecipeMap						(null, "gt.recipe.cruciblealloying"				, "Combination Smelting"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 6, 6, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Temperature: "		,    1, " K"	, T, T, F, T);
		public static final RecipeMap		sGenerifierRecipes							= new RecipeMap						(null, "gt.recipe.generifier"					, "Generifier"						, null							, 0, 1, RES_PATH_GUI+"machines/Generifier"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 1, 1, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		
		public static final RecipeMap		sSharpeningRecipes							= new RecipeMap						(null, "gt.recipe.sharpener"					, "Sharpener"						, null							, 0, 1, RES_PATH_GUI+"machines/Sharpener"						,/*IN-OUT-MIN-ITEM=*/ 1, 2, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sSifterRecipes								= new RecipeMap						(null, "gt.recipe.sifter"						, "Sifter"							, null							, 2, 1, RES_PATH_GUI+"machines/Sifter"							,/*IN-OUT-MIN-ITEM=*/ 1,12, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sHammerRecipes								= new RecipeMapHammer				(null, "gt.recipe.hammer"						, "Hammer"							, null							, 6, 3, RES_PATH_GUI+"machines/Hammer"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, T, T);
		public static final RecipeMap		sChiselRecipes								= new RecipeMapChisel				(null, "gt.recipe.chisel"						, "Chisel"							, null							, 0, 1, RES_PATH_GUI+"machines/Chisel"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, T, T);
		public static final RecipeMap		sShredderRecipes							= new RecipeMapShredder				(null, "gt.recipe.shredder"						, "Shredder"						, null							, 0, 1, RES_PATH_GUI+"machines/Shredder"						,/*IN-OUT-MIN-ITEM=*/ 1, 6, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCrusherRecipes								= new RecipeMapCrusher				(null, "gt.recipe.crusher"						, "Crusher"							, null							, 0, 1, RES_PATH_GUI+"machines/Crusher"							,/*IN-OUT-MIN-ITEM=*/ 1, 4, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sLatheRecipes								= new RecipeMap						(null, "gt.recipe.lathe"						, "Lathe"							, null							, 0, 1, RES_PATH_GUI+"machines/Lathe"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCutterRecipes								= new RecipeMapCutter				(null, "gt.recipe.cutter"						, "Cutter"							, null							, 0, 1, RES_PATH_GUI+"machines/Cutter"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 1,/*IN-OUT-MIN-FLUID=*/ 1, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCoagulatorRecipes							= new RecipeMap						(null, "gt.recipe.coagulator"					, "Coagulator"						, null							, 0, 1, RES_PATH_GUI+"machines/Coagulator"						,/*IN-OUT-MIN-ITEM=*/ 0, 1, 0,/*IN-OUT-MIN-FLUID=*/ 1, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sSqueezerRecipes							= new RecipeMap						(null, "gt.recipe.squeezer"						, "Squeezer"						, null							, 0, 1, RES_PATH_GUI+"machines/Squeezer"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 1, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sJuicerRecipes								= new RecipeMap						(null, "gt.recipe.juicer"						, "Juicer"							, null							, 0, 1, RES_PATH_GUI+"machines/Juicer"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 1, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, T, T);
		public static final RecipeMap		sCompressorRecipes							= new RecipeMap						(null, "gt.recipe.compressor"					, "Compressor"						, null							, 0, 1, RES_PATH_GUI+"machines/Compressor"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCentrifugeRecipes							= new RecipeMap						(null, "gt.recipe.centrifuge"					, "Centrifuge"						, null							, 0, 1, RES_PATH_GUI+"machines/Centrifuge"						,/*IN-OUT-MIN-ITEM=*/ 1, 6, 0,/*IN-OUT-MIN-FLUID=*/ 1, 6, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sElectrolyzerRecipes						= new RecipeMap						(null, "gt.recipe.electrolyzer"					, "Electrolyzer"					, null							, 0, 1, RES_PATH_GUI+"machines/Electrolyzer"					,/*IN-OUT-MIN-ITEM=*/ 1, 6, 0,/*IN-OUT-MIN-FLUID=*/ 1, 6, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sRollingMillRecipes							= new RecipeMap						(null, "gt.recipe.rollingmill"					, "Rolling Mill"					, null							, 0, 1, RES_PATH_GUI+"machines/RollingMill"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sRollBenderRecipes							= new RecipeMap						(null, "gt.recipe.rollbender"					, "Roll Bender"						, null							, 0, 1, RES_PATH_GUI+"machines/RollBender"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sRollFormerRecipes							= new RecipeMap						(null, "gt.recipe.rollformer"					, "Roll Former"						, null							, 0, 1, RES_PATH_GUI+"machines/RollFormer"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sClusterMillRecipes							= new RecipeMap						(null, "gt.recipe.clustermill"					, "Cluster Mill"					, null							, 0, 1, RES_PATH_GUI+"machines/ClusterMill"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sWiremillRecipes							= new RecipeMap						(null, "gt.recipe.wiremill"						, "Wiremill"						, null							, 0, 1, RES_PATH_GUI+"machines/Wiremill"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sMixerRecipes								= new RecipeMap						(null, "gt.recipe.mixer"						, "Mixer"							, null							, 0, 1, RES_PATH_GUI+"machines/Mixer"							,/*IN-OUT-MIN-ITEM=*/ 6, 1, 0,/*IN-OUT-MIN-FLUID=*/ 6, 2, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCannerRecipes								= new RecipeMapFluidCanner			(null, "gt.recipe.canner"						, "Canning Machine"					, null							, 0, 1, RES_PATH_GUI+"machines/Canner"							,/*IN-OUT-MIN-ITEM=*/ 2, 2, 1,/*IN-OUT-MIN-FLUID=*/ 1, 1, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sInjectorRecipes							= new RecipeMap						(null, "gt.recipe.injector"						, "Injector"						, null							, 0, 1, RES_PATH_GUI+"machines/Injector"						,/*IN-OUT-MIN-ITEM=*/ 2, 1, 0,/*IN-OUT-MIN-FLUID=*/ 2, 1, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sRoastingRecipes							= new RecipeMap						(null, "gt.recipe.roaster"						, "Roaster"							, null							, 0, 1, RES_PATH_GUI+"machines/Roaster"							,/*IN-OUT-MIN-ITEM=*/ 1, 3, 1,/*IN-OUT-MIN-FLUID=*/ 1, 1, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sDryingRecipes								= new RecipeMap						(null, "gt.recipe.drying"						, "Dryer"							, null							, 0, 1, RES_PATH_GUI+"machines/Dryer"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 1, 1, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sFermenterRecipes							= new RecipeMap						(null, "gt.recipe.fermenter"					, "Fermenter"						, null							, 0, 1, RES_PATH_GUI+"machines/Fermenter"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 1, 1, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sDistilleryRecipes							= new RecipeMap						(null, "gt.recipe.distillery"					, "Distillery"						, null							, 0, 1, RES_PATH_GUI+"machines/Distillery"						,/*IN-OUT-MIN-ITEM=*/ 1, 2, 1,/*IN-OUT-MIN-FLUID=*/ 1, 2, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sExtruderRecipes							= new RecipeMap						(null, "gt.recipe.extruder"						, "Extruder"						, null							, 0, 1, RES_PATH_GUI+"machines/Extruder"						,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sPolarizerRecipes							= new RecipeMap						(null, "gt.recipe.polarizer"					, "Polarizer"						, null							, 0, 1, RES_PATH_GUI+"machines/Polarizer"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sLoomRecipes								= new RecipeMap						(null, "gt.recipe.loom"							, "Loom"							, null							, 0, 1, RES_PATH_GUI+"machines/Loom"							,/*IN-OUT-MIN-ITEM=*/ 6, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCookingRecipes								= new RecipeMap						(null, "gt.recipe.cooker"						, "Cooker"							, null							, 0, 1, RES_PATH_GUI+"machines/Cooker"							,/*IN-OUT-MIN-ITEM=*/ 9, 1, 1,/*IN-OUT-MIN-FLUID=*/ 3, 1, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sPressRecipes								= new RecipeMapFormingPress			(null, "gt.recipe.press"						, "Press"							, null							, 0, 1, RES_PATH_GUI+"machines/Press"							,/*IN-OUT-MIN-ITEM=*/ 3, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sBathRecipes								= new RecipeMapBath					(null, "gt.recipe.bath"							, "Bath"							, null							, 0, 1, RES_PATH_GUI+"machines/Bath"							,/*IN-OUT-MIN-ITEM=*/ 6, 6, 1,/*IN-OUT-MIN-FLUID=*/ 1, 3, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sSmelterRecipes								= new RecipeMap						(null, "gt.recipe.smelter"						, "Smelter"							, null							, 0, 1, RES_PATH_GUI+"machines/Smelter"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 1, 1, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sLaserEngraverRecipes						= new RecipeMap						(null, "gt.recipe.laserengraver"				, "Precision Laser Engraver"		, null							, 0, 1, RES_PATH_GUI+"machines/LaserEngraver"					,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sWelderRecipes								= new RecipeMap						(null, "gt.recipe.welder"						, "Welding Machine"					, null							, 0, 1, RES_PATH_GUI+"machines/Welder"							,/*IN-OUT-MIN-ITEM=*/ 6, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCrystallisationCrucibleRecipes				= new RecipeMap						(null, "gt.recipe.crystallisationcrucible"		, "Crystallisation Crucible"		, null							, 0, 1, RES_PATH_GUI+"machines/CrystallisationCrucible"			,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 2, 0, 1,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sScannerVisualsRecipes						= new RecipeMapScannerVisuals		(null, "gt.recipe.scannervisuals"				, "Scanner (Visuals)"				, null							, 0, 1, RES_PATH_GUI+"machines/ScannerVisuals"					,/*IN-OUT-MIN-ITEM=*/ 2, 2, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sPrinterRecipes								= new RecipeMapPrinter				(null, "gt.recipe.printer"						, "Printer"							, null							, 0, 1, RES_PATH_GUI+"machines/Printer"							,/*IN-OUT-MIN-ITEM=*/ 2, 1, 1,/*IN-OUT-MIN-FLUID=*/ 6, 0, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sSluiceRecipes								= new RecipeMap						(null, "gt.recipe.sluice"						, "Sluice"							, null							, 0, 1, RES_PATH_GUI+"machines/Sluice"							,/*IN-OUT-MIN-ITEM=*/ 1, 9, 1,/*IN-OUT-MIN-FLUID=*/ 1, 1, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sMagneticSeparatorRecipes					= new RecipeMap						(null, "gt.recipe.magneticseparator"			, "Magnetic Separator"				, null							, 0, 1, RES_PATH_GUI+"machines/MagneticSeparator"				,/*IN-OUT-MIN-ITEM=*/ 1, 6, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sAutocrafterRecipes							= new RecipeMapAutocrafting			(null, "gt.recipe.autocrafting"					, "Crafting"						, null							, 0, 1, RES_PATH_GUI+"machines/Crafting"						,/*IN-OUT-MIN-ITEM=*/ 9,12, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sMassfabRecipes								= new RecipeMap						(null, "gt.recipe.massfab"						, "Matter Fabricator"				, null							, 0, 1, RES_PATH_GUI+"machines/Massfab"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sScannerMolecularRecipes					= new RecipeMapScannerMolecular		(null, "gt.recipe.scannermolecular"				, "Molecular Scanner"				, null							, 0, 1, RES_PATH_GUI+"machines/ScannerMolecular"				,/*IN-OUT-MIN-ITEM=*/ 2, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sReplicatorRecipes							= new RecipeMapReplicator			(null, "gt.recipe.replicator"					, "Matter Replicator"				, null							, 0, 1, RES_PATH_GUI+"machines/Replicator"						,/*IN-OUT-MIN-ITEM=*/ 3, 3, 1,/*IN-OUT-MIN-FLUID=*/ 3, 3, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		
		public static final RecipeMap		sAutoclaveRecipes							= new RecipeMap						(null, "gt.recipe.autoclave"					, "Autoclave"						, null							, 0, 1, RES_PATH_GUI+"machines/Autoclave"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sBrewingRecipes								= new RecipeMap						(null, "gt.recipe.brewer"						, "Brewing Machine"					, null							, 0, 1, RES_PATH_GUI+"machines/PotionBrewer"					,/*IN-OUT-MIN-ITEM=*/ 1, 0, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sFluidSolidficationRecipes					= new RecipeMap						(null, "gt.recipe.fluidsolidifier"				, "Fluid Solidifier"				, null							, 0, 1, RES_PATH_GUI+"machines/FluidSolidifier"					,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sBoxinatorRecipes							= new RecipeMap						(null, "gt.recipe.packager"						, "Packager"						, null							, 0, 1, RES_PATH_GUI+"machines/Packager"						,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sUnboxinatorRecipes							= new RecipeMapUnboxinator			(null, "gt.recipe.unpackager"					, "Unpackager"						, null							, 0, 1, RES_PATH_GUI+"machines/Unpackager"						,/*IN-OUT-MIN-ITEM=*/ 1, 2, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sFusionRecipes								= new RecipeMap						(null, "gt.recipe.fusionreactor"				, "Fusion Reactor"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 2, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Start: "				,    1, " EU"	, T, F/*F*/, F/*T*/, T);
		public static final RecipeMap		sBlastRecipes								= new RecipeMap						(null, "gt.recipe.blastfurnace"					, "Blast Furnace"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 2, 2, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Heat Capacity: "		,    1, " K"	, F, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sImplosionRecipes							= new RecipeMap						(null, "gt.recipe.implosioncompressor"			, "Implosion Compressor"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 2, 2, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sVacuumRecipes								= new RecipeMap						(null, "gt.recipe.vacuumfreezer"				, "Vacuum Freezer"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sAssemblerRecipes							= new RecipeMapAssembler			(null, "gt.recipe.assembler"					, "Assembler"						, null							, 0, 1, RES_PATH_GUI+"machines/Assembler"						,/*IN-OUT-MIN-ITEM=*/ 2, 1, 1,/*IN-OUT-MIN-FLUID=*/ 1, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sCNCRecipes									= new RecipeMap						(null, "gt.recipe.cncmachine"					, "CNC Machine"						, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 1, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sSlicerRecipes								= new RecipeMap						(null, "gt.recipe.slicer"						, "Slicer"							, null							, 0, 1, RES_PATH_GUI+"machines/Slicer"							,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sAmplifiers									= new RecipeMap						(null, "gt.recipe.uuamplifier"					, "UU Amplifier"					, null							, 0, 1, RES_PATH_GUI+"machines/Amplifabricator"					,/*IN-OUT-MIN-ITEM=*/ 1, 0, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		
		public static final RecipeMap		sFuelsBurn									= new RecipeMapFuel					(null, "gt.recipe.fuels.burn"					, "Burnable Fuels"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsGas									= new RecipeMapFuel					(null, "gt.recipe.fuels.gas"					, "Gas Fuels"						, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsHot									= new RecipeMapFuel					(null, "gt.recipe.fuels.hot"					, "Hot Fuels"						, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsPlasma								= new RecipeMapFuel					(null, "gt.recipe.fuels.plasma"					, "Plasma Fuels"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsEngine								= new RecipeMapFuel					(null, "gt.recipe.fuels.engine"					, "Engine Fuels"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsTurbine								= new RecipeMapFuel					(null, "gt.recipe.fuels.turbine"				, "Turbine Fuels"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsMagic									= new RecipeMapFuel					(null, "gt.recipe.fuels.magic"					, "Magic Fuels"						, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		
		// Old Fuel Maps
		public static final RecipeMapFuel	sDieselFuels								= new RecipeMapFuel					(null, "gt.recipe.dieselgeneratorfuel"			, "Diesel Generator Fuel"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		public static final RecipeMapFuel	sTurbineFuels								= new RecipeMapFuel					(null, "gt.recipe.gasturbinefuel"				, "Gas Turbine Fuel"				, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		public static final RecipeMapFuel	sHotFuels									= new RecipeMapFuel					(null, "gt.recipe.thermalgeneratorfuel"			, "Thermal Generator Fuel"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*F*/, F/*T*/, F);
		public static final RecipeMapFuel	sDenseLiquidFuels							= new RecipeMapFuel					(null, "gt.recipe.semifluidboilerfuels"			, "Semifluid Boiler Fuels"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		public static final RecipeMapFuel	sPlasmaFuels								= new RecipeMapFuel					(null, "gt.recipe.plasmageneratorfuels"			, "Plasma generator Fuels"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*F*/, F/*T*/, F);
		public static final RecipeMapFuel	sMagicFuels									= new RecipeMapFuel					(null, "gt.recipe.magicfuels"					, "Magic Fuels"						, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*F*/, F/*T*/, F);
		public static final RecipeMapFuel	sSmallNaquadahReactorFuels					= new RecipeMapFuel					(null, "gt.recipe.smallnaquadahreactor"			, "Small Naquadah Reactor"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		public static final RecipeMapFuel	sLargeNaquadahReactorFuels					= new RecipeMapFuel					(null, "gt.recipe.largenaquadahreactor"			, "Large Naquadah Reactor"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		public static final RecipeMapFuel	sFluidNaquadahReactorFuels					= new RecipeMapFuel					(null, "gt.recipe.fluidnaquadahreactor"			, "Fluid Naquadah Reactor"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		
		/** List of Machines that can perform/use the Recipes of this Map. */
		public final List<ItemStack> mRecipeMachineList = new ArrayListNoNulls<ItemStack>();
		/** HashMap of Recipes based on their Items */
		public final Map<ItemStackContainer, Collection<Recipe>> mRecipeItemMap = new ItemStackMap<ItemStackContainer, Collection<Recipe>>();
		/** HashMap of Recipes based on their Fluids */
		public final Map<String, Collection<Recipe>> mRecipeFluidMap = new HashMap<String, Collection<Recipe>>();
		/** The List of all Recipes */
		public final Collection<Recipe> mRecipeList;
		/** String used as an unlocalised Name. */
		public final String mNameInternal;
		/** String used as a localised Name in things that shouldn't be localised, like Config File Names. */
		public final String mNameLocal, mNameLocalUnderscored;
		/** String used in NEI for the Recipe Lists. If null it will use the unlocalised Name instead*/
		public final String mNameNEI;
		/** GUI used for NEI Display. Usually the GUI of the Machine itself */
		public final String mGUIPath;
		public final String mNEISpecialValuePre, mNEISpecialValuePost;
		public final byte mProgressBarDirection, mProgressBarAmount;
		public final int mInputItemsCount, mOutputItemsCount, mInputFluidCount, mOutputFluidCount, mMinimalInputItems, mMinimalInputFluids, mMinimalInputs;
		public final long mNEISpecialValueMultiplier, mPower;
		public final boolean mNEIAllowed, mShowVoltageAmperageInNEI, mNeedsOutputs;
		/** Used to determine Input Tank Size. Contains the size of the largest FluidStack Input. */
		public int mMaxFluidInputSize = 1000;
		/** The Config File corresponding to this Recipe Handler. Will be initialised by GT_API. */
		public Config mConfigFile = null;
		
		/**
		 * Initialises a new type of Recipe Handler.
		 * @param aRecipeList a List you specify as Recipe List. Usually just an ArrayList with a pre-initialised Size.
		 * @param aNameInternal the unlocalised Name of this Recipe Handler, used mainly for NEI.
		 * @param aNameLocal the displayed Name inside the NEI Recipe GUI and regular ToolTips.
		 * @param aNEIGUIPath the displayed GUI Texture, usually just a Machine GUI. Auto-Attaches ".png" if forgotten.
		 * @param aInputItemsCount the usual amount of Input Slots this Recipe Class has.
		 * @param aOutputItemsCount the usual amount of Output Slots this Recipe Class has.
		 * @param aNEISpecialValuePre the String in front of the Special Value in NEI.
		 * @param aNEISpecialValueMultiplier the Value the Special Value is getting Multiplied with before displaying
		 * @param aNEISpecialValuePost the String after the Special Value. Usually for a Unit or something.
		 * @param aNEIAllowed if NEI is allowed to display this Recipe Handler in general.
		 */
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			mNEIAllowed = aNEIAllowed;
			mShowVoltageAmperageInNEI = aShowVoltageAmperageInNEI;
			mNeedsOutputs = aNeedsOutputs;
			mRecipeList = (aRecipeList == null ? new HashSetNoNulls<Recipe>() : aRecipeList);
			mNameInternal = aNameInternal;
			mNameLocal = aNameLocal;
			mNameLocalUnderscored = mNameLocal.replaceAll(" ", "_");
			mNameNEI = aNameNEI == null ? mNameInternal : aNameNEI;
			mGUIPath = aNEIGUIPath.endsWith(".png")?aNEIGUIPath:aNEIGUIPath + ".png";
			mNEISpecialValuePre = aNEISpecialValuePre;
			mNEISpecialValueMultiplier = aNEISpecialValueMultiplier;
			mNEISpecialValuePost = aNEISpecialValuePost;
			mPower = aPower;
			mMinimalInputItems = (int)aMinimalInputItems;
			mInputItemsCount = (int)Math.max(aInputItemsCount, mMinimalInputItems);
			mOutputItemsCount = (int)aOutputItemsCount;
			mMinimalInputFluids = (int)aMinimalInputFluids;
			mInputFluidCount = (int)Math.max(aInputFluidCount, mMinimalInputFluids);
			mOutputFluidCount = (int)aOutputFluidCount;
			mMinimalInputs = (int)aMinimalInputs;
			mProgressBarDirection = (byte)aProgressBarDirection;
			mProgressBarAmount = (byte)aProgressBarAmount;
			LH.add(mNameInternal, mNameLocal);
			if (RECIPE_MAPS.containsKey(mNameInternal)) throw new IllegalArgumentException("Recipe Map Name already exists: " + mNameInternal);
			RECIPE_MAPS.put(mNameInternal, this);
			if (aConfigAllowed) if (GAPI.mBeforePreInit != null) GAPI.mBeforePreInit.add(this); else run();
		}
		
		@Override public String toString() {return mNameInternal;}
		@Override public void run() {mConfigFile = new Config(DirectoriesGT.CONFIG_RECIPES, mNameLocalUnderscored+".cfg");}
		
		@Deprecated
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			this(aRecipeList, aNameInternal, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, 0, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Deprecated
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed) {
			this(aRecipeList, aNameInternal, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, 0, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, T);
		}
		
		@Deprecated
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed) {
			this(aRecipeList, aNameInternal, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, 0, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, T, T);
		}
		
		@Deprecated
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aMinimalInputFluids, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed) {
			this(aRecipeList, aNameInternal, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, 0, 0, aMinimalInputFluids, 0, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, T, T);
		}
		
		@Deprecated
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aMinimalInputFluids, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed) {
			this(aRecipeList, aNameInternal, aNameLocal, aNameNEI, 0, 1, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, 0, 0, aMinimalInputFluids, 0, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, T, T);
		}
		
		public Recipe addRecipe(boolean aOptimize, ItemStack[] aInputs, ItemStack[] aOutputs, Object aSpecial, long[] aOutputChances, FluidStack[] aFluidInputs, FluidStack[] aFluidOutputs, long aDuration, long aEUt, long aSpecialValue) {
			return addRecipe(new Recipe(aOptimize, T, aInputs, aOutputs, aSpecial, aOutputChances, aFluidInputs, aFluidOutputs, aDuration, aEUt, aSpecialValue));
		}
		
		public Recipe addRecipe(Recipe aRecipe) {
			return addRecipe(aRecipe, T, F, F, T);
		}
		
		// The Number next to the addRecipe is for the amount of Item Inputs used. X = Array
		
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput																					, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2																, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs																				, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput																					, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2																, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs																				, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput																					, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2																, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs																				, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long   aChance												, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration																, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long[] aChances												, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration																, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput							, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, aInputs								, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long   aChance												, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long[] aChances												, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, aInputs								, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long   aChance												, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long[] aChances												, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput																					, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2																, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs																				, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput																					, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2																, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs																				, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput																					, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2																, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs																				, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance												, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration																, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances												, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration																, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput							, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance												, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances												, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance												, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances												, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		
		public Recipe addRecipe(Recipe aRecipe, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors) {
			aRecipe.mHidden = aHidden;
			aRecipe.mFakeRecipe = aFakeRecipe;
			if (aCheckForCollisions && findRecipe(null, null, F, Long.MAX_VALUE, null, aRecipe.mFluidInputs, aRecipe.mInputs) != null) return null;
			return add(aRecipe, aLogErrors);
		}
		
		public Recipe addRecipe(Recipe aRecipe, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden) {
			return addRecipe(aRecipe, aCheckForCollisions, aFakeRecipe, aHidden, T);
		}
		
		/** Only used for fake Recipe Handlers to show something in NEI, do not use this for adding actual Recipes! findRecipe wont find fake Recipes, containsInput WILL find fake Recipes */
		public Recipe addFakeRecipe(boolean aCheckForCollisions, ItemStack[] aInputs, ItemStack[] aOutputs, Object aSpecial, long[] aOutputChances, FluidStack[] aFluidInputs, FluidStack[] aFluidOutputs, long aDuration, long aEUt, long aSpecialValue) {
			return addFakeRecipe(aCheckForCollisions, new Recipe(F, F, aInputs, aOutputs, aSpecial, aOutputChances, aFluidInputs, aFluidOutputs, aDuration, aEUt, aSpecialValue));
		}
		
		/** Only used for fake Recipe Handlers to show something in NEI, do not use this for adding actual Recipes! findRecipe wont find fake Recipes, containsInput WILL find fake Recipes */
		public Recipe addFakeRecipe(boolean aCheckForCollisions, ItemStack[] aInputs, ItemStack[] aOutputs, Object aSpecial, FluidStack[] aFluidInputs, FluidStack[] aFluidOutputs, long aDuration, long aEUt, long aSpecialValue) {
			return addFakeRecipe(aCheckForCollisions, new Recipe(F, F, aInputs, aOutputs, aSpecial, null, aFluidInputs, aFluidOutputs, aDuration, aEUt, aSpecialValue));
		}
		
		/** Only used for fake Recipe Handlers to show something in NEI, do not use this for adding actual Recipes! findRecipe wont find fake Recipes, containsInput WILL find fake Recipes */
		public Recipe addFakeRecipe(boolean aCheckForCollisions, Recipe aRecipe) {
			return addRecipe(aRecipe, aCheckForCollisions, T, F, T);
		}
		
		public Recipe add(Recipe aRecipe) {
			return add(aRecipe, T);
		}
		
		public synchronized Recipe add(Recipe aRecipe, boolean aLogErrors) {
			if (!aRecipe.mFakeRecipe) {
				boolean tErrored = F, tFailed = F;
				if (aRecipe.mInputs.length + aRecipe.mFluidInputs.length <= 0) {
					if (aLogErrors) ERR.println("Error: Recipe has no Inputs!");
					tFailed = T;
				} else {
					if (mNeedsOutputs && aRecipe.mOutputs.length + aRecipe.mFluidOutputs.length <= 0) {if (aLogErrors) ERR.println("Error: Recipe has no Outputs!"											); tFailed = T;}
					if (aRecipe.mInputs			.length < mMinimalInputItems						) {if (aLogErrors) ERR.println("Error: Recipe has less than the minimal amount of Input ItemStacks!"		); tFailed = T;}
					if (aRecipe.mFluidInputs	.length < mMinimalInputFluids						) {if (aLogErrors) ERR.println("Error: Recipe has less than the minimal amount of Input FluidStacks!"	); tFailed = T;}
					if (aRecipe.mFluidInputs.length + aRecipe.mInputs.length < mMinimalInputs		) {if (aLogErrors) ERR.println("Error: Recipe has less than the minimal amount of general Inputs!"		); tFailed = T;}
					if (aRecipe.mInputs			.length > mInputItemsCount							) {if (aLogErrors) ERR.println("Error: Recipe has more than the maximum amount of Input ItemStacks!"		); tFailed = T;}
					if (aRecipe.mFluidInputs	.length > mInputFluidCount							) {if (aLogErrors) ERR.println("Error: Recipe has more than the maximum amount of Input FluidStacks!"	); tFailed = T;}
				}
				if (aRecipe.mOutputs			.length > mOutputItemsCount							) {if (aLogErrors) ERR.println("WARNING: Recipe has more than the maximum amount of Output ItemStacks!"	); tErrored = T;}
				if (aRecipe.mFluidOutputs		.length > mOutputFluidCount							) {if (aLogErrors) ERR.println("WARNING: Recipe has more than the maximum amount of Output FluidStacks!"	); tErrored = T;}
				if (aRecipe.mDuration															== 0) {if (aLogErrors) ERR.println("WARNING: Recipe has no Duration Value!"									); tErrored = T;}
				
				if (tErrored || tFailed) {
					if (aLogErrors) {int i = 0; for (StackTraceElement tElement : new Exception().getStackTrace()) if (!tElement.getClassName().equals("gregapi.recipes.Recipe$RecipeMap")) if (i++<5 && !tElement.getClassName().startsWith("sun")) ERR.println("\tat " + tElement); else break;}
					if (tFailed) return null;
				}
			}
			
			if (mConfigFile != null) {
				String tConfigName = "";
				if (aRecipe.mInputs.length > 0) tConfigName += UT.Stacks.configNames(aRecipe.mInputs);
				if (aRecipe.mFluidInputs.length > 0) tConfigName += UT.Fluids.configNames(aRecipe.mFluidInputs);
				if (UT.Code.stringValid(tConfigName)) {
					aRecipe.mDuration = mConfigFile.get(mNameLocalUnderscored, tConfigName, aRecipe.mEnabled?aRecipe.mDuration:0);
					aRecipe.mEnabled = (aRecipe.mDuration > 0);
				}
			}
			
			if (!mRecipeList.add(aRecipe)) return null;
			for (FluidStack aFluid : aRecipe.mFluidInputs) if (aFluid != null) {
				mMaxFluidInputSize = Math.max(mMaxFluidInputSize, aFluid.amount);
				Collection<Recipe> tList = mRecipeFluidMap.get(aFluid.getFluid().getName());
				if (tList == null) mRecipeFluidMap.put(aFluid.getFluid().getName(), tList = new HashSet<Recipe>(1));
				tList.add(aRecipe);
			}
			return addToItemMap(aRecipe);
		}
		
		public void reInit() {
        	mRecipeItemMap.clear();
        	for (Recipe tRecipe : mRecipeList) {
        		OreDictManager.INSTANCE.setStackArray(T, tRecipe.mInputs);
        		OreDictManager.INSTANCE.setStackArray(T, tRecipe.mOutputs);
            	addToItemMap(tRecipe);
        	}
		}
		
		public Slot_Normal getSpecialSlot(ITileEntityInventoryGUI aInventory, int aIndex, int aX, int aY) {
			return null;
		}
		
		/** @return if this Item is a valid Input for any for the Recipes */
		public boolean containsInput(ItemStack aStack, ItemStack aSpecialSlot) {
			return aStack != null && (mRecipeItemMap.containsKey(new ItemStackContainer(aStack)) || mRecipeItemMap.containsKey(new ItemStackContainer(UT.Stacks.copyMeta(W, aStack))));
		}
		
		/** @return if this Fluid is a valid Input for any for the Recipes */
		public boolean containsInput(FluidStack aFluid, ItemStack aSpecialSlot) {
			return aFluid != null && containsInput(aFluid.getFluid(), aSpecialSlot);
		}
		
		/** @return if this Fluid is a valid Input for any for the Recipes */
		public boolean containsInput(Fluid aFluid, ItemStack aSpecialSlot) {
			return aFluid != null && mRecipeFluidMap.containsKey(aFluid.getName());
		}
		
		private Recipe oRecipe = null;
		
		/**
		 * finds a Recipe matching the aFluid and ItemStack Inputs.
		 * @param aTileEntity an Object representing the current coordinates of the executing Block/Entity/Whatever. This may be null, especially during Startup.
		 * @param aRecipe in case this is != null it will try to use this Recipe first when looking things up.
		 * @param aNotUnificated if this is T the Recipe searcher will unificate the ItemStack Inputs
		 * @param aSize Voltage of the Machine or Long.MAX_VALUE if it has no Voltage
		 * @param aFluids the Fluid Inputs
		 * @param aSpecialSlot the content of the Special Slot, the regular Manager doesn't do anything with this, but some custom ones do.
		 * @param aInputs the Item Inputs
		 * @return the Recipe it has found or null for no matching Recipe
		 */
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			// No Recipes? Well, nothing to be found then.
			if (mRecipeList.isEmpty()) return null;
			
			// Some Recipe Classes require a certain amount of Inputs of certain kinds. Like "at least 1 Fluid + 1 Stack" or "at least 2 Stacks" before they start searching for Recipes.
			// This improves Performance massively, especially if people leave things like Circuits, Molds or Shapes in their Machines to select Sub Recipes.
			if (GAPI_POST.mFinishedServerStarted > 0) {
				long tItemAmount = UT.Code.getNonNulls(aInputs);
				if (tItemAmount < mMinimalInputItems) return null;
				long tFluidAmount = UT.Code.getNonNulls(aFluids);
				if (tFluidAmount < mMinimalInputFluids) return null;
				if (tFluidAmount + tItemAmount < mMinimalInputs) return null;
			}
			
			// Unification happens here in case the Input isn't already unificated.
			if (aNotUnificated) aInputs = OreDictManager.INSTANCE.getStackArray(T, (Object[])aInputs);
			
			// Check the Recipe which has been used last time in order to not have to search for it again, if possible.
			if (aRecipe != null) if (!aRecipe.mFakeRecipe && aRecipe.mCanBeBuffered && aRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return aRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, aRecipe.mEUt)?oRecipe=aRecipe:null;
			if (oRecipe != null) if (!oRecipe.mFakeRecipe && oRecipe.mCanBeBuffered && oRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return oRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, oRecipe.mEUt)?oRecipe:null;
			
			// Now look for the Recipes inside the Item HashMaps, but only when the Recipes usually have Items.
			if (mInputItemsCount > 0 && aInputs != null) for (ItemStack tStack1 : aInputs) if (tStack1 != null) {
				Collection<Recipe>
				tRecipes = mRecipeItemMap.get(new ItemStackContainer(tStack1));
				if (tRecipes != null) for (Recipe tRecipe : tRecipes) if (!tRecipe.mFakeRecipe && tRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return tRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, tRecipe.mEUt)?oRecipe=tRecipe:null;
				
				tRecipes = mRecipeItemMap.get(new ItemStackContainer(UT.Stacks.copyMeta(W, tStack1)));
				if (tRecipes != null) for (Recipe tRecipe : tRecipes) if (!tRecipe.mFakeRecipe && tRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return tRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, tRecipe.mEUt)?oRecipe=tRecipe:null;
				
				ItemStack tStack2 = OreDictManager.INSTANCE.getStack_(F, tStack1);
				
				if (!UT.Stacks.equal(tStack1, tStack2, true)) {
				tRecipes = mRecipeItemMap.get(new ItemStackContainer(tStack2));
				if (tRecipes != null) for (Recipe tRecipe : tRecipes) if (!tRecipe.mFakeRecipe && tRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return tRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, tRecipe.mEUt)?oRecipe=tRecipe:null;
				}
				if (tStack1.getItem() != tStack2.getItem()) {
				tRecipes = mRecipeItemMap.get(new ItemStackContainer(UT.Stacks.copyMeta(W, tStack2)));
				if (tRecipes != null) for (Recipe tRecipe : tRecipes) if (!tRecipe.mFakeRecipe && tRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return tRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, tRecipe.mEUt)?oRecipe=tRecipe:null;
				}
			}
			
			// If the minimal Amount of Items for the Recipe is 0, then it could be a Fluid-Only Recipe, so check that Map too.
			if (mInputFluidCount > 0 && mMinimalInputItems == 0 && aFluids != null) for (FluidStack aFluid : aFluids) if (aFluid != null) {
				Collection<Recipe>
				tRecipes = mRecipeFluidMap.get(aFluid.getFluid().getName());
				if (tRecipes != null) for (Recipe tRecipe : tRecipes) if (!tRecipe.mFakeRecipe && tRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return tRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, tRecipe.mEUt)?oRecipe=tRecipe:null;
			}
			
			// And nothing has been found.
			return null;
		}
		
		public List<Recipe> getNEIAllRecipes() {
			ArrayListNoNulls<Recipe> rList = new ArrayListNoNulls<Recipe>();
			for (Recipe tRecipe : mRecipeList) if (!tRecipe.mHidden) rList.add(tRecipe);
			return rList;
		}
		
		public List<Recipe> getNEIRecipes(ItemStack... aOutputs) {
			ArrayListNoNulls<Recipe> rList = new ArrayListNoNulls<Recipe>();
			for (Recipe tRecipe : mRecipeList) if (!tRecipe.mHidden) {
				for (ItemStack aOutput : aOutputs) if (aOutput != null) {
					if (IL.Display_Fluid.equal(aOutput, T, T)) {
						for (FluidStack tOutput : tRecipe.mFluidOutputs) {
							if (Fluids.id(tOutput) == UT.Stacks.meta(aOutput)) {
								rList.add(tRecipe);
								break;
							}
						}
					} else {
						for (ItemStack tOutput : tRecipe.mOutputs) if (tOutput != null && tOutput.getItem() == aOutput.getItem()) {
							if (UT.Stacks.meta(tOutput) == W || UT.Stacks.meta(tOutput) == UT.Stacks.meta(aOutput) || tOutput.isItemStackDamageable()) {
								rList.add(tRecipe);
								break;
							}
						}
					}
				}
			}
			return rList;
		}
		
		public List<Recipe> getNEIUsages(ItemStack... aInputs) {
			ArrayListNoNulls<Recipe> rList = new ArrayListNoNulls<Recipe>();
			for (Recipe tRecipe : mRecipeList) if (!tRecipe.mHidden) {
				for (ItemStack aInput : aInputs) if (aInput != null) {
					if (IL.Display_Fluid.equal(aInput, T, T)) {
						for (FluidStack tInput : tRecipe.mFluidInputs) {
							if (Fluids.id(tInput) == UT.Stacks.meta(aInput)) {
								rList.add(tRecipe);
								break;
							}
						}
					} else {
						for (ItemStack tInput : tRecipe.mInputs) if (tInput != null && tInput.getItem() == aInput.getItem()) {
							if (UT.Stacks.meta(tInput) == W || UT.Stacks.meta(tInput) == UT.Stacks.meta(aInput) || tInput.isItemStackDamageable()) {
								rList.add(tRecipe);
								break;
							}
						}
					}
				}
			}
			return rList;
		}
		
		public Recipe addToItemMap(Recipe aRecipe) {
			for (ItemStack aStack : aRecipe.mInputs) if (aStack != null) {
				ItemStackContainer tStack = new ItemStackContainer(aStack);
				Collection<Recipe> tList = mRecipeItemMap.get(tStack);
				if (tList == null) mRecipeItemMap.put(tStack, tList = new HashSet<Recipe>(1));
				tList.add(aRecipe);
			}
			return aRecipe;
		}
		
		public boolean openNEI() {
			try {codechicken.nei.recipe.GuiCraftingRecipe.openRecipeGui(mNameNEI, new Object[0]); return true;} catch(Throwable e) {/**/}
			return false;
		}
	}
	
	public static void reInit() {
        for (RecipeMap tMapEntry : RecipeMap.RECIPE_MAPS.values()) tMapEntry.reInit();
	}
	
	/** If you want to change the Output, feel free to modify or even replace the whole ItemStack Array, for Inputs, please add a new Recipe, because of the HashMaps. */
	public ItemStack[] mInputs, mOutputs;
	/** If you want to change the Output, feel free to modify or even replace the whole ItemStack Array, for Inputs, please add a new Recipe, because of the HashMaps. */
	public FluidStack[] mFluidInputs, mFluidOutputs;
	/** If you changed the amount of Array-Items inside the Output Array then the length of this Array must be larger or equal to the Output Array. A chance of 10000 equals 100% if the MaxChance Value is default. */
	public long[] mChances, mMaxChances;
	/** An Item that needs to be inside the Special Slot, like for example the Copy Slot inside the Printer. This is only useful for Fake Recipes in NEI, since findRecipe() and containsInput() don't give a shit about this Field. Lists are also possible. */
	public Object mSpecialItems;
	
	public long mDuration, mEUt, mSpecialValue;
	
	/** Use this to just disable a specific Recipe, but the Configuration enables that already for every single Recipe. */
	public boolean mEnabled = T;
	/** If this Recipe is hidden from NEI */
	public boolean mHidden = F;
	/** If this Recipe is Fake and therefore doesn't get found by the findRecipe Function (It is still in the HashMaps, so that containsInput does return T on those fake Inputs) */
	public boolean mFakeRecipe = F;
	/** If this Recipe can be stored inside a Machine in order to make Recipe searching more Efficient by trying the previously used Recipe first. In case you have a Recipe Map overriding things and returning one time use Recipes, you have to set this to F. */
	public boolean mCanBeBuffered = T;
	/** If this Recipe needs the Output Slots to be completely empty. Needed in case you have randomised Outputs */
	public boolean mNeedsEmptyOutput = F;
	
	public int getOutputChance(long aIndex) {if (aIndex < 0 || aIndex >= mChances.length) return 10000; return (int)mChances[(int)aIndex];}
	public int getMaxChance(long aIndex) {if (aIndex < 0 || aIndex >= mMaxChances.length) return 10000; return (int)mMaxChances[(int)aIndex];}
	
	public ItemStack getRepresentativeInput(long aIndex) {if (aIndex < 0 || aIndex >= mInputs.length) return null; return UT.Stacks.copy(mInputs[(int)aIndex]);}
	public ItemStack getOutput(long aIndex) {if (aIndex < 0 || aIndex >= mOutputs.length) return null; return UT.Stacks.copy(mOutputs[(int)aIndex]);}
	
	public FluidStack getRepresentativeFluidInput(long aIndex) {if (aIndex < 0 || aIndex >= mFluidInputs.length || mFluidInputs[(int)aIndex] == null) return null; return mFluidInputs[(int)aIndex].copy();}
	public FluidStack getFluidOutput(long aIndex) {if (aIndex < 0 || aIndex >= mFluidOutputs.length || mFluidOutputs[(int)aIndex] == null) return null; return mFluidOutputs[(int)aIndex].copy();}
	
	public Recipe copy() {
		return new Recipe(this);
	}
	
	public FluidStack[] getFluidOutputs(Random aRandom) {
		FluidStack[] rArray = new FluidStack[mFluidOutputs.length];
		for (int i = 0; i < rArray.length; i++) rArray[i] = getFluidOutput(i);
		return rArray;
	}
	
	public ItemStack[] getOutputs(Random aRandom) {
		ItemStack[] rArray = new ItemStack[mOutputs.length];
		if (aRandom == null) aRandom = new Random();
		for (int i = 0; i < rArray.length; i++) {
			ItemStack tOutput = getOutput(i);
			if (tOutput != null) {
				int tChance = getOutputChance(i);
				if (tChance > 0) {
					int tMax = getMaxChance(i);
					if (tChance == tMax) {
						rArray[i] = tOutput;
					} else if (tChance < tMax) {
						for (int j = 0; j < tOutput.stackSize; j++) if (aRandom.nextInt(tMax) < tChance) {
							if (rArray[i] == null) rArray[i] = UT.Stacks.amount(1, tOutput); else rArray[i].stackSize++;
						}
					} else {
						int tIterations = (3*tChance) / (2*tMax), tAmount = tOutput.stackSize;
						tChance /= tIterations;
						for (int j = 0; j < tIterations; j++) if (aRandom.nextInt(tMax) < tChance) {
							if (rArray[i] == null) rArray[i] = tOutput; else rArray[i].stackSize += tAmount;
						}
					}
				} else {
					rArray[i] = tOutput;
				}
			}
		}
		return rArray;
	}
	
	@Deprecated
	public boolean isRecipeInputEqual(boolean aDecreaseStacksizeBySuccess, FluidStack[] aFluidInputs, ItemStack... aInputs) {
		return isRecipeInputEqual(aDecreaseStacksizeBySuccess, F, aFluidInputs, aInputs);
	}
	
	public boolean isRecipeInputEqual(boolean aDecreaseStacksizeBySuccess, boolean aDontCheckStackSizes, FluidStack[] aFluidInputs, ItemStack... aInputs) {
		if (mFluidInputs.length > 0 && (aFluidInputs == null || aFluidInputs.length < 1)) return F;		
		if (mInputs.length > 0 && (aInputs == null || aInputs.length < 1)) return F;
		
		for (FluidStack tFluid : mFluidInputs) if (tFluid != null) {
			boolean temp = T;
			for (FluidStack aFluid : aFluidInputs) if (aFluid != null && aFluid.isFluidEqual(tFluid) && (aDontCheckStackSizes || aFluid.amount >= tFluid.amount)) {temp = F; break;}
			if (temp) return F;
		}
		
		for (ItemStack tStack : mInputs) if (UT.Stacks.valid(tStack)) {boolean temp = T;
			for (ItemStack aStack : aInputs) if (UT.Stacks.valid(aStack) && OreDictManager.INSTANCE.equal_(F, aStack, tStack) && (aDontCheckStackSizes || aStack.stackSize >= tStack.stackSize)) {temp = F; break;}
			if (temp) return F;
		}
		
		if (aDecreaseStacksizeBySuccess) {
			if (aFluidInputs	!= null) for (FluidStack	tFluid : mFluidInputs	) if (tFluid != null)			for (FluidStack	aFluid : aFluidInputs	) if (aFluid != null && aFluid.isFluidEqual(tFluid)												&& (aDontCheckStackSizes || aFluid.amount		>= tFluid.amount	)) {aFluid.amount		-= tFluid.amount	; break;}
			if (aInputs			!= null) for (ItemStack		tStack : mInputs		) if (UT.Stacks.valid(tStack))	for (ItemStack	aStack : aInputs		) if (UT.Stacks.valid(aStack) && OreDictManager.INSTANCE.equal_(F, aStack, tStack)	&& (aDontCheckStackSizes || aStack.stackSize	>= tStack.stackSize	)) {aStack.stackSize	-= tStack.stackSize	; break;}
		}
		
		return T;
	}
	
	private Recipe(Recipe aRecipe) {
		mInputs = UT.Stacks.copyArray((Object[])aRecipe.mInputs);
		mOutputs = UT.Stacks.copyArray((Object[])aRecipe.mOutputs);
		mSpecialItems = aRecipe.mSpecialItems;
		mChances = Arrays.copyOf(aRecipe.mChances, aRecipe.mChances.length);
		mMaxChances = Arrays.copyOf(aRecipe.mMaxChances, aRecipe.mMaxChances.length);
		mFluidInputs = UT.Fluids.copyArray(aRecipe.mFluidInputs);
		mFluidOutputs = UT.Fluids.copyArray(aRecipe.mFluidOutputs);
		mEUt = aRecipe.mEUt;
		mDuration = aRecipe.mDuration;
		mSpecialValue = aRecipe.mSpecialValue;
		mNeedsEmptyOutput = aRecipe.mNeedsEmptyOutput;
		mCanBeBuffered = aRecipe.mCanBeBuffered;
		mFakeRecipe = aRecipe.mFakeRecipe;
		mEnabled = aRecipe.mEnabled;
		mHidden = aRecipe.mHidden;
	}
	
	public Recipe(boolean aOptimize, boolean aUnificate, ItemStack[] aInputs, ItemStack[] aOutputs, Object aSpecialItems, long[] aChances, FluidStack[] aFluidInputs, FluidStack[] aFluidOutputs, long aDuration, long aEUt, long aSpecialValue) {
		if (aInputs       == null) aInputs       = ZL_ITEMSTACK;
		if (aOutputs      == null) aOutputs      = ZL_ITEMSTACK;
		if (aFluidInputs  == null) aFluidInputs  = ZL_FLUIDSTACK;
		if (aFluidOutputs == null) aFluidOutputs = ZL_FLUIDSTACK;
		if (aChances      == null) aChances      = aOutputs.length<0?ZL_LONG:new long[aOutputs.length];
		if (aChances.length < aOutputs.length) aChances = Arrays.copyOf(aChances, aOutputs.length);
		aInputs  = UT.Code.getWithoutTrailingNulls(aInputs ).toArray(ZL_ITEMSTACK);
		aOutputs = UT.Code.getWithoutTrailingNulls(aOutputs).toArray(ZL_ITEMSTACK);
		if (aUnificate) {
			OreDictManager.INSTANCE.setStackArray_(T, aInputs );
			OreDictManager.INSTANCE.setStackArray_(T, aOutputs);
		}
		aFluidInputs  = UT.Code.getWithoutNulls(aFluidInputs ).toArray(ZL_FLUIDSTACK);
		aFluidOutputs = UT.Code.getWithoutNulls(aFluidOutputs).toArray(ZL_FLUIDSTACK);
		
		for (ItemStack tStack : aOutputs) {
			if (tStack != null) {
				if (UT.Stacks.meta(tStack) == W) UT.Stacks.meta(tStack, 0);
				UT.Stacks.update(tStack);
			}
		}
		
		for (int i = 0; i < aChances		.length; i++) if (aChances[i] <=  0) aChances[i] = 10000;
		for (int i = 0; i < aInputs			.length; i++) if (aInputs [i] != NI) aInputs [i] = aInputs [i].copy();
		for (int i = 0; i < aOutputs		.length; i++) if (aOutputs[i] != NI) aOutputs[i] = aOutputs[i].copy();
		for (int i = 0; i < aFluidInputs	.length; i++) aFluidInputs [i] = aFluidInputs [i].copy();
		for (int i = 0; i < aFluidOutputs	.length; i++) aFluidOutputs[i] = aFluidOutputs[i].copy();
		
		if (aOptimize)  {
			for (int i = 0; i < aInputs.length; i++) if (aInputs[i] != null && UT.Stacks.meta(aInputs[i]) != W) for (int j = 0; j < aOutputs.length; j++) {
				if (UT.Stacks.equal(aInputs[i], aOutputs[j])) {
					if (aInputs[i].stackSize >= aOutputs[j].stackSize) {
						aInputs[i].stackSize -= aOutputs[j].stackSize;
						aOutputs[j] = null;
					} else {
						aOutputs[j].stackSize -= aInputs[i].stackSize;
					}
				}
			}
			
			if (aDuration >= 32) {
				ArrayList<ItemStack> tList = new ArrayListNoNulls<ItemStack>(Arrays.asList(aInputs));
				tList.addAll(Arrays.asList(aOutputs));
				
				for (byte i = (byte)Math.min(64, aDuration / 16); i > 1; i--) if (aDuration / i >= 16) {
					boolean temp = T;
					for (int j = 0, k = tList.size(); temp &&  j < k; j++) if (tList.get(j).stackSize  % i != 0) temp = F;
					for (int j = 0; temp && j < aFluidInputs .length; j++) if (aFluidInputs [j].amount % i != 0) temp = F;
					for (int j = 0; temp && j < aFluidOutputs.length; j++) if (aFluidOutputs[j].amount % i != 0) temp = F;
					if (temp) {
						for (int j = 0, k = tList.size();  j < k; j++) tList.get(j).stackSize  /= i;
						for (int j = 0; j < aFluidInputs .length; j++) aFluidInputs [j].amount /= i;
						for (int j = 0; j < aFluidOutputs.length; j++) aFluidOutputs[j].amount /= i;
						aDuration /= i;
						break;
					}
				}
			}
		}
		
		mInputs = aInputs;
		mOutputs = aOutputs;
		mSpecialItems = aSpecialItems;
		mChances = aChances;
		Arrays.fill(mMaxChances = new long[aChances.length], 10000);
		mFluidInputs = aFluidInputs;
		mFluidOutputs = aFluidOutputs;
		mDuration = aDuration;
		mSpecialValue = aSpecialValue;
		mEUt = aEUt;
		
//		checkCellBalance();
	}
}